/*
  +----------------------------------------------------------------------+
  | PHP Version 5                                                        |
  +----------------------------------------------------------------------+
  | Copyright (c) 1997-2005 The PHP Group                                |
  +----------------------------------------------------------------------+
  | This source file is subject to version 3.0 of the PHP license,       |
  | that is bundled with this package in the file LICENSE, and is        |
  | available through the world-wide-web at the following url:           |
  | http://www.php.net/license/3_0.txt.                                  |
  | If you did not receive a copy of the PHP license and are unable to   |
  | obtain it through the world-wide-web, please send a note to          |
  | license@php.net so we can mail you a copy immediately.               |
  +----------------------------------------------------------------------+
  | Author: Wez Furlong <wez@omniti.com>                                 |
  +----------------------------------------------------------------------+
*/

/* $Id: dtrace.c,v 1.5 2005/08/09 02:41:23 wez Exp $ */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "php.h"
#include "php_ini.h"
#include "ext/standard/info.h"
#include "php_dtrace.h"
#include "zend_execute.h"

#include <dtrace.h>
#include <sys/sdt.h>


/* {{{ dtrace_functions[] */
function_entry dtrace_functions[] = {
	{NULL, NULL, NULL}
};
/* }}} */

/* {{{ dtrace_module_entry */
zend_module_entry dtrace_module_entry = {
#if ZEND_MODULE_API_NO >= 20010901
	STANDARD_MODULE_HEADER,
#endif
	"dtrace",
	dtrace_functions,
	PHP_MINIT(dtrace),
	PHP_MSHUTDOWN(dtrace),
	NULL,
	NULL,
	PHP_MINFO(dtrace),
	"1.0.2",
	STANDARD_MODULE_PROPERTIES
};
/* }}} */

#ifdef COMPILE_DL_DTRACE
ZEND_GET_MODULE(dtrace)
#endif

static void (*ye_olde_execute)(zend_op_array *op_array TSRMLS_DC);
static void (*ye_olde_execute_internal)(zend_execute_data *execute_data_ptr,
	int return_value_used TSRMLS_DC);

void php_dtrace_execute_internal(zend_execute_data *execute_data_ptr,
	int return_value_used TSRMLS_DC)
{
	char *fname = NULL;
	
	fname = get_active_function_name(TSRMLS_CC);;
   	DTRACE_PROBE3(php, function__entry, fname, zend_get_executed_filename(TSRMLS_C),
		zend_get_executed_lineno(TSRMLS_C));
	if (ye_olde_execute_internal) {
		ye_olde_execute_internal(execute_data_ptr, return_value_used TSRMLS_CC);
	} else {
		execute_internal(execute_data_ptr, return_value_used TSRMLS_CC);
	}
	DTRACE_PROBE3(php, function__return, fname, zend_get_executed_filename(TSRMLS_C),
		zend_get_executed_lineno(TSRMLS_C));
}

void php_dtrace_execute(zend_op_array *op_array TSRMLS_DC)
{
	char *fname = NULL;
	
	fname = get_active_function_name(TSRMLS_CC);;
   	DTRACE_PROBE3(php, function__entry, fname, zend_get_executed_filename(TSRMLS_C),
		zend_get_executed_lineno(TSRMLS_C));
	ye_olde_execute(op_array TSRMLS_CC);
	DTRACE_PROBE3(php, function__return, fname, zend_get_executed_filename(TSRMLS_C),
		zend_get_executed_lineno(TSRMLS_C));
}


/* {{{ PHP_MINIT_FUNCTION */
PHP_MINIT_FUNCTION(dtrace)
{
	ye_olde_execute = zend_execute;
	ye_olde_execute_internal = zend_execute_internal;
	zend_execute = php_dtrace_execute;
	zend_execute_internal = php_dtrace_execute_internal;
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MSHUTDOWN_FUNCTION */
PHP_MSHUTDOWN_FUNCTION(dtrace)
{
	zend_execute = ye_olde_execute;
	zend_execute_internal = ye_olde_execute_internal;
	return SUCCESS;
}
/* }}} */

/* {{{ PHP_MINFO_FUNCTION */
PHP_MINFO_FUNCTION(dtrace)
{
	php_info_print_table_start();
	php_info_print_table_header(2, "dtrace support", "enabled");
	php_info_print_table_end();
}
/* }}} */


/*
 * Local variables:
 * tab-width: 4
 * c-basic-offset: 4
 * End:
 * vim600: noet sw=4 ts=4 fdm=marker
 * vim<600: noet sw=4 ts=4
 */
